/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.editor.name.operation;

import java.util.Set;

import org.eclipse.core.commands.ExecutionException;
import org.eclipse.core.commands.operations.IUndoContext;
import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;

import eu.etaxonomy.cdm.model.name.HomotypicalGroup;
import eu.etaxonomy.cdm.model.name.TaxonNameBase;
import eu.etaxonomy.cdm.model.taxon.SynonymRelationship;
import eu.etaxonomy.cdm.model.taxon.Taxon;
import eu.etaxonomy.cdm.model.taxon.TaxonRelationship;
import eu.etaxonomy.cdm.model.taxon.TaxonRelationshipType;
import eu.etaxonomy.taxeditor.operation.AbstractPostOperation;
import eu.etaxonomy.taxeditor.operation.IPostOperationEnabled;
import eu.etaxonomy.taxeditor.store.StoreUtil;

/**
 * <p>ChangeConceptToSynonymOperation class.</p>
 *
 * @author p.ciardelli
 * @author n.hoffmann
 * @created 15.01.2009
 * @version 1.0
 */
public class ChangeConceptToSynonymOperation extends
		AbstractPostOperation {
	
	private Taxon concept;

	private HomotypicalGroup homotypicalGroup;

	private TaxonRelationship taxonRelationship;
	private TaxonRelationshipType oldRelationshipType;
	
	private SynonymRelationship newSynonymRelationship;

	/**
	 * <p>Constructor for ChangeConceptToSynonymOperation.</p>
	 *
	 * @param label a {@link java.lang.String} object.
	 * @param undoContext a {@link org.eclipse.core.commands.operations.IUndoContext} object.
	 * @param taxon a {@link eu.etaxonomy.cdm.model.taxon.Taxon} object.
	 * @param concept a {@link eu.etaxonomy.cdm.model.taxon.Taxon} object.
	 * @param homotypicalGroup a {@link eu.etaxonomy.cdm.model.name.HomotypicalGroup} object.
	 * @param editor a {@link eu.etaxonomy.taxeditor.operation.IPostOperationEnabled} object.
	 */
	public ChangeConceptToSynonymOperation(String label,
			IUndoContext undoContext, Taxon taxon, Taxon concept, HomotypicalGroup homotypicalGroup,
			IPostOperationEnabled editor) {
		super(label, undoContext, taxon, editor);
		
		Set<TaxonRelationship> taxonRelationships = taxon.getTaxonRelations(concept);
		
		if(taxonRelationships.size() > 1){
			StoreUtil.warningDialog("Multiple relations between taxa", this, "There are multiple relations between the " +
					"accepted and the related taxon. This case is not handled by the software yet");
			return;
		}
		
		this.taxonRelationship = taxonRelationships.iterator().next();
		this.oldRelationshipType = taxonRelationship.getType();
		
		this.concept = concept;
		this.homotypicalGroup = homotypicalGroup != null ? homotypicalGroup : HomotypicalGroup.NewInstance();
	}

	/* (non-Javadoc)
	 * @see org.eclipse.core.commands.operations.AbstractOperation#execute(org.eclipse.core.runtime.IProgressMonitor, org.eclipse.core.runtime.IAdaptable)
	 */
	/** {@inheritDoc} */
	@Override
	public IStatus execute(IProgressMonitor monitor, IAdaptable info)
			throws ExecutionException {
		
		// Create new synonym using concept name
		TaxonNameBase<?, ?> synonymName = concept.getName();
		
		// Remove concept relation from taxon
		taxon.removeTaxon(concept, oldRelationshipType);
		monitor.worked(20);
        
		// Add name to new homotypic group
		homotypicalGroup.addTypifiedName(synonymName);
		monitor.worked(40);
		
        // Create a new synonym for the taxon
		newSynonymRelationship = taxon.addHeterotypicSynonymName(synonymName);
			
		return postExecute(newSynonymRelationship.getSynonym());
	}

	/* (non-Javadoc)
	 * @see org.eclipse.core.commands.operations.AbstractOperation#redo(org.eclipse.core.runtime.IProgressMonitor, org.eclipse.core.runtime.IAdaptable)
	 */
	/** {@inheritDoc} */
	@Override
	public IStatus redo(IProgressMonitor monitor, IAdaptable info)
			throws ExecutionException {
		// TODO redo for change misapplied name ...
		return null;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.core.commands.operations.AbstractOperation#undo(org.eclipse.core.runtime.IProgressMonitor, org.eclipse.core.runtime.IAdaptable)
	 */
	/** {@inheritDoc} */
	@Override
	public IStatus undo(IProgressMonitor monitor, IAdaptable info)
			throws ExecutionException {
		// TODO undo for change misapplied name ...
		return null;
	}
}
